###############################################################################
# Functions for eQTL Weighted Genome-wide Association Study
#------------------------------------------------------------------------------ 
# Author: Lin Li (ll323@cornell.edu)
# Last updated: April 21, 2013
###############################################################################

#------------------------------------------------------------------------------
# Function:
#     ewgwas()
# Description:
#     calculate binary and general weights 
# Input:
#     chr: chromosome
#     pos: coordinates (based on NCBI36)
#     pval: original p-value
#     database: Rdata containing the cis eQTL information
#     alpha: type I error, to be passed to b1.wt()
#     power: power for up-weighted hypotheses, to be passed to b1.wt()
# Output:
#     a data frame containing chr, pos, and pval that are same as input, 
#     and the following eQTL weighted p-values
#     wb: binary weight
#     wg: general weight

#-- Function b1.wt() -------------------------------------------------------------
# Function:
#     b1.wt()
# Description:
#     calculate the two parameters for binary weights to maximize minimum power
# Input:
#     m: number of hypotheses
#     epsilon: fraction of hypotheses with high power(=1-beta), i.e. up-weighted
#     alpha: type I error
#     power: power for up-weighted hypotheses
# Output:
#     w1: weight for upweighted hypotheses
#     w0: weight for hypotheses that are not upweighted
#------------------------------------------------------------------------------
b1.wt<-function(m, epsilon, alpha, power) {
  beta=1-power
  C=pnorm( qnorm(alpha/m,lower.tail=F) + qnorm(1-beta,lower.tail=F), lower.tail=F )
  B=C*m*(1-epsilon)/(alpha-epsilon*C*m)
  w1=B/(epsilon*B+1-epsilon)
  w0=1/(epsilon*B+1-epsilon)
  return(list(wt1=w1,wt0=w0))
}

#-- Function ewgwas() ----------------------------------------------------------
ewgwas<-function(chr,pos,pval,database='eqtldb_mrca_cis1m.Rdata',alpha=0.05,power=0.6) {
  id=paste('chr',chr,':',pos,sep='')
  if(file.exists(database)) load(database) else stop(paste(database,' does not exist!',sep=''))
  
  index.eqtl=match(names(eqtl.db.cis1m),id)
  wt.eqtl=rep(1,length(id))
  wt.eqtl[index.eqtl[!is.na(index.eqtl)]]=eqtl.db.cis1m[!is.na(index.eqtl)]
  
  bwt=b1.wt(length(wt.eqtl),mean(wt.eqtl>1),alpha,power)
  wb=(wt.eqtl>1)*bwt$wt1+(wt.eqtl==1)*bwt$wt0
  wb=wb/mean(wb)
  Qb=pval/wb
  
  wg=sqrt(wt.eqtl)
  wg=wg/mean(wg)
  Qg=pval/wg
  
  return(data.frame(chr=chr,pos=pos,wb=wb,wg=wg,pval=pval,Qb=Qb,Qg=Qg))
}



